"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createProvider = createProvider;
exports.combineProviders = combineProviders;
exports.useProvider = useProvider;

var _react = _interopRequireWildcard(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj["default"] = obj; return newObj; } }

function _toArray(arr) { return _arrayWithHoles(arr) || _iterableToArray(arr) || _nonIterableRest(); }

function _iterableToArray(iter) { if (Symbol.iterator in Object(iter) || Object.prototype.toString.call(iter) === "[object Arguments]") return Array.from(iter); }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance"); }

function _iterableToArrayLimit(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

var contextMap = new Map();

function createProvider() {
  var initialState = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : null;
  var reducer = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : function (state) {
    return state;
  };
  var StateContext = (0, _react.createContext)(null);
  var DispatchContext = (0, _react.createContext)(null);

  var Provider = function Provider(_ref) {
    var children = _ref.children;

    var _useReducer = (0, _react.useReducer)(reducer, initialState),
        _useReducer2 = _slicedToArray(_useReducer, 2),
        state = _useReducer2[0],
        dispatch = _useReducer2[1];

    return _react["default"].createElement(DispatchContext.Provider, {
      value: dispatch
    }, _react["default"].createElement(StateContext.Provider, {
      value: state
    }, children));
  };

  Provider.propTypes = {
    children: _propTypes["default"].node
  };
  contextMap.set(Provider, {
    StateContext: StateContext,
    DispatchContext: DispatchContext
  });
  return Provider;
}

function combineProviders(providers) {
  var _providers = _toArray(providers),
      first = _providers[0],
      rest = _providers.slice(1);

  return function Combined(_ref2) {
    var children = _ref2.children;
    return rest.reduce(function (Combined, Provider) {
      return _react["default"].createElement(Combined, null, _react["default"].createElement(Provider, null, children));
    }, first);
  };
}

function useProvider(Provider) {
  var _ref3 = contextMap.get(Provider) || {},
      _ref3$StateContext = _ref3.StateContext,
      StateContext = _ref3$StateContext === void 0 ? {} : _ref3$StateContext,
      _ref3$DispatchContext = _ref3.DispatchContext,
      DispatchContext = _ref3$DispatchContext === void 0 ? {} : _ref3$DispatchContext;

  var state = (0, _react.useContext)(StateContext);
  var dispatch = (0, _react.useContext)(DispatchContext);

  if (state === undefined || dispatch === undefined) {
    console.error("useProvider must be called with a Provider existent in the component's tree");
  }

  return [state, dispatch];
}